/*******************************************************************************
* FILE NAME: user_routines_fast.c <EDU VERSION>
*
* DESCRIPTION:
*  This file is where the user can add their custom code within the framework
*  of the routines below. 
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
* OPTIONS:  Interrupts are disabled and not used by default.
*
* CHANGES:  Combined beacon, receiver, and tracker in one package
*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "beacon.h"
#include "receiver.h"
#include "tracker.h"

/*** DEFINE USER VARIABLES AND INITIALIZE THEM HERE ***/


/*******************************************************************************
* FUNCTION NAME: InterruptVectorLow
* PURPOSE:       Low priority interrupt vector
* CALLED FROM:   nowhere by default
* ARGUMENTS:     none
* RETURNS:       void
* DO NOT MODIFY OR DELETE THIS FUNCTION 
*******************************************************************************/
#pragma code InterruptVectorLow = LOW_INT_VECTOR
void InterruptVectorLow (void)
{
  _asm
    goto InterruptHandlerLow  /*jump to interrupt routine*/
  _endasm
}


/*******************************************************************************
* FUNCTION NAME: InterruptHandlerLow
* PURPOSE:       Low priority interrupt handler
* If you want to use these external low priority interrupts or any of the
* peripheral interrupts then you must enable them in your initialization
* routine.  Innovation First, Inc. will not provide support for using these
* interrupts, so be careful.
* CALLED FROM:   this file, InterruptVectorLow routine
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
#pragma code
#pragma interruptlow InterruptHandlerLow

void InterruptHandlerLow ()     
{                               
  unsigned char int_byte;       
  unsigned char Port_B;
  unsigned char Port_B_Delta;       
  static unsigned char Old_Port_B = 0xFF; // state of port b the last time
  	
  /*** Test Internal Interrupts **/
  if (INTCONbits.TMR0IF) // timer 0 Interrupt
  {
  	INTCONbits.TMR0IF = 0; // clear the interrupt flag
	Timer_0_Int_Handler(); // beacon state machine interrupt handler
  }
  else if (PIR1bits.TMR1IF) // timer 1 interrupt - used by the IR receiver
  {
	PIR1bits.TMR1IF = 0; // clear the interrupt flag
	Timer_1_Int_Handler(); // call the timer 1 interrupt handler (in receiver.c)
  }  

  /*** Test External Interupts **/


	if (INTCON3bits.INT2IF) // external interrupt 1 - used by IR sensor 1
	{ 
		INTCON3bits.INT2IF = 0; // clear the interrupt flag
		Int_1_Handler(); // call the IR sensor 1 interrupt handler (in receiver.c)
	}

	else if (INTCON3bits.INT3IF) // external interrupt 2 - used by IR sensor 2
	{
		INTCON3bits.INT3IF = 0; // clear the interrupt flag
		Int_2_Handler(); // call the IR sensor 2 interrupt handler (in receiver.c)
	}

	else if (INTCONbits.RBIF) // external interrupts 3 through 6
	{
		Port_B = PORTB; // remove the "mismatch condition" by reading port b            
		INTCONbits.RBIF = 0; // clear the interrupt flag
		Port_B_Delta = Port_B ^ Old_Port_B; // determine which bits have changed
		Old_Port_B = Port_B; // save a copy of port b for next time around
	 
		if(Port_B_Delta & 0x10) // did external interrupt 3 change state?
		{
			// if used, call the interrupt 3 handler here
		}

		if(Port_B_Delta & 0x20) // did external interrupt 4 change state? - IR sensor 3
		{
			Int_4_Handler(Port_B & 0x20 ? 1 : 0); // call the interrupt 4 handler (in receiver.c)
		}

		if(Port_B_Delta & 0x40) // did external interrupt 5 change state? - IR sensor 4
		{
			Int_5_Handler(Port_B & 0x40 ? 1 : 0); // call the interrupt 5 handler (in receiver.c)
		}
		
		if(Port_B_Delta & 0x80) // did external interrupt 6 change state?
		{
			// if used, call the interrupt 6 handler here
		}
	}

}


/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Local_IO
* PURPOSE:       Execute user's realtime code.
* You should modify this routine by adding code which you wish to run fast.
* It will be executed every program loop, and not wait for fresh data 
* from the master processor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Local_IO(void)
{
	static unsigned int Old_Clock = 0;

	if (Clock > Old_Clock) // stay synchronized to beacon data updates (this is important)
	{
		Track_Beacon(LEFT); // allow the left beacon tracker state machine to run
		Track_Beacon(RIGHT); // allow the right beacon tracker state machine to run
		
		if ((Tracker_Data[LEFT].Status == BOTH_IN_VIEW) &&
			(Tracker_Data[RIGHT].Status == BOTH_IN_VIEW))
		{
			// both trackers have an angular solution; calculate beacon vector
			// here using the known baseline distance between the trackers and
			// angles derived from the two Tracker_Data[].Position variables

			// calculate new motor PWM values here

		}
	// finally, update the servo motor positions
	LEFT_TRACKER_SERVO = Tracker_Data[LEFT].Position;
	RIGHT_TRACKER_SERVO = Tracker_Data[RIGHT].Position;
	
	Old_Clock = Clock; // take a snapshot of the clock
	} 
}


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
